import datasets


BASE_URL = 'http://dl.turkunlp.org/ptm'

TAG_LABELS = datasets.features.ClassLabel(names=[
    'O',
    'B-Chemical',
    'B-Complex',
    'B-Family',
    'B-Protein',
    'I-Chemical',
    'I-Complex',
    'I-Family',
    'I-Protein',
])


class PTMdata(datasets.GeneratorBasedBuilder):
    def _info(self):
        return datasets.DatasetInfo(
            description="PTM corpus NER annotations",
            features=datasets.Features({
                'id': datasets.Value('string'),
                'tokens': datasets.Sequence(datasets.Value('string')),
                'ner_tags': datasets.Sequence(TAG_LABELS),
            }),
            supervised_keys=None,
        )

    def _split_generators(self, dl_manager):
        urls = { s: f'{BASE_URL}/{s}.conll' for s in ('train', 'dev', 'test') }
        files = dl_manager.download(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={'filename': files['train']}
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={'filename': files['dev']}
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={'filename': files['test']}
            ),
        ]

    def _generate_examples(self, filename):
        sentence_counter = 0

        with open(filename) as f:
            current_words = []
            current_labels = []
            for row in f:
                row = row.rstrip()
                row_split = row.split("\t")
                if len(row_split) == 2:
                    token, label = row_split
                    current_words.append(token)
                    current_labels.append(label)
                else:
                    if not current_words:
                        continue
                    assert len(current_words) == len(current_labels)
                    sentence = (
                        sentence_counter,
                        {
                            'id': str(sentence_counter),
                            'tokens': current_words,
                            'ner_tags': current_labels,
                        },
                    )
                    sentence_counter += 1
                    current_words = []
                    current_labels = []
                    yield sentence

            # if something remains:
            if current_words:
                sentence = (
                    sentence_counter,
                    {
                        'id': str(sentence_counter),
                        'tokens': current_words,
                        'ner_tags': current_labels,
                    },
                )
                yield sentence
